import os
import xlrd, xlwt
import openpyxl
import datetime

from tqdm import tqdm


def generate_xls(filepath: str, worksheet_data: dict):
    datetime_str = datetime.datetime.now().strftime('%Y-%m-%d_%H%M%S')
    new_filepath = filepath.replace('.xls', '_Split_{}.xls'.format(datetime_str))
    new_workbook = xlwt.Workbook(encoding='utf-8')
    for worksheet_name, row_data_list in worksheet_data.items():
        new_worksheet = new_workbook.add_sheet(worksheet_name)
        for row_index, row_data in enumerate(row_data_list):
            for column_index, data in enumerate(row_data):
                new_worksheet.write(row_index, column_index, data)
    new_workbook.save(new_filepath)
    return new_filepath


def process_xls(filepath, column: int, worksheet_name: str = None):
    try:
        workbook = xlrd.open_workbook(filepath, formatting_info=True)
    except:
        return "文件读取异常：{}".format(filepath)
    if worksheet_name:
        worksheet = workbook.sheet_by_name(worksheet_name)
    else:
        worksheet = workbook.sheet_by_index(0)
    rows = worksheet.nrows
    cols = worksheet.ncols
    split_data_dict = {}
    for r in tqdm(range(rows)):
        row_data = [worksheet.cell(r, c).value if worksheet.cell(r, c).value else ' ' for c in range(cols)]
        temp_data = row_data[column - 1]
        temp_data_list = split_data_dict.get(temp_data, [])
        temp_data_list.append(row_data)
        split_data_dict[temp_data] = temp_data_list
    new_filepath = generate_xls(filepath, split_data_dict)
    return "数据保存在新文件中，文件名：{}".format(new_filepath)


def generate_xlsx(filepath: str, worksheet_data: dict):
    datetime_str = datetime.datetime.now().strftime('%Y-%m-%d_%H%M%S')
    new_filepath = filepath.replace('.xlsx', '_Split_{}.xlsx'.format(datetime_str))
    new_workbook = openpyxl.Workbook()
    for worksheet_name, row_data_list in worksheet_data.items():
        new_worksheet = new_workbook.create_sheet(worksheet_name)
        for row_data in row_data_list:
            new_worksheet.append(row_data)
    new_workbook.save(new_filepath)
    return new_filepath


def process_xlsx(filepath: str, column: int, worksheet_name: str = None):
    try:
        workbook = openpyxl.load_workbook(filepath, read_only=True, data_only=True)
    except:
        return "文件读取异常：{}".format(filepath)
    if worksheet_name:
        worksheet = workbook.get_sheet_by_name(worksheet_name)
    else:
        worksheet = workbook.active
    if worksheet.max_column < column:
        return "最大列数是{}，取不到第{}列".format(worksheet.max_column, column)

    split_data_dict = {}
    for row in worksheet.rows:
        row_data = [cell.value if cell.value else ' ' for cell in row]
        temp_data = row_data[column - 1]
        temp_data_list = split_data_dict.get(temp_data, [])
        temp_data_list.append(row_data)
        split_data_dict[temp_data] = temp_data_list
    new_filepath = generate_xlsx(filepath, split_data_dict)
    return "数据保存在新文件中，文件名：{}".format(new_filepath)


def split_excel_by_column(filepath: str, column: int, worksheet_name: str = None):
    if filepath.endswith('.xlsx'):
        result = process_xlsx(filepath, column, worksheet_name)
    elif filepath.endswith('.xls'):
        result = process_xls(filepath, column, worksheet_name)
    else:
        print("文件格式不对，不进行处理")
        return "文件格式不对，不进行处理"
    print(result)
    return result


if __name__ == "__main__":
    filename = 'sedemo.xls'
    # filename = 'SEdemo.xlsx'
    result = split_excel_by_column(filename, 6)  # 处理文件，表格的第六列，worksheet_name指定工作表，不指定则读取文件默认工作表
    print(result)



# -*- coding: UTF-8 -*-
'''
@作者 ：B站/抖音/微博/小红书/公众号，都叫：程序员晚枫
@微信 ：CoderWanFeng : https://mp.weixin.qq.com/s/Nt8E8vC-ZsoN1McTOYbY2g
@个人网站 ：www.python-office.com
@Date    ：2023/3/22 20:49 
@Description     ：
'''
import os
from pathlib import Path

import pandas as pd
from poprogress import simple_progress


def query4excel(query_content, query_path, output_path, output_name):

    if not output_name.endswith('xlsx') and not output_name.endswith('xls'):
        print('output_name必须以.xlsx或者.xls结尾')
        return
    abs_query_path = Path(query_path).absolute()
    if output_path:
        abs_output_path = Path(output_path).absolute() / output_name
    else:
        abs_output_path = abs_query_path / output_name
    pwd = abs_output_path.parent
    # t = type(pwd)
    if not pwd.exists():
        pwd.mkdir()
    waiting_query_excel_files = []
    # 如果不存在，则不做处理
    if not abs_query_path.exists():
        print("path does not exist path = " + query_path)
        return
    # 判断是否是文件
    elif abs_query_path.is_file():
        print("path file type is file " + query_path)
        waiting_query_excel_files.append(query_path)
    # 如果是目录，则遍历目录下面的文件
    elif abs_query_path.is_dir():
        for dirpath, dirnames, filenames in simple_progress(os.walk(str(abs_query_path)),
                                                            desc=f'正在查找：{query_path}：'):
            for filename in filenames:
                if filename.endswith('.xlsx') or filename.endswith('.xls'):
                    waiting_query_excel_files.append(Path(dirpath).absolute() / filename)
    res_df = pd.DataFrame()
    print(f'{query_path}下，一共有{str(len(waiting_query_excel_files))}个Excel')
    for excel in simple_progress(waiting_query_excel_files, desc='正在搜索Excel中符合条件的数据：'):
        df = pd.read_excel(excel, sheet_name=None, header=None)
        for sheet in df.values():
            for i, current_row in enumerate(sheet.itertuples()):
                # if i == 0:
                #     print(sheet.iloc[i].to_dict())
                if query_content in current_row:
                    file_dict = {"文件位置":str(excel)}
                    file_dict.update(sheet.iloc[i].to_dict())
                    current_row_df = pd.DataFrame(file_dict, index=[0])
                    res_df = res_df._append(current_row_df)
                    # print()
    # print(res_df)
    res_df.to_excel(str(abs_output_path),index=False)
    print(f'完成搜索，结果存储在：{str(abs_output_path)}')




import os
from pathlib import Path

import pandas as pd
import xlwings as xw
from faker import Faker
from openpyxl import load_workbook
from pofile import get_files
from pofile import mkdir
from poprogress import simple_progress
from tqdm import tqdm

# 忽略waring警告
from poexcel.lib import pandas_mem
from poexcel.lib.excel import SplitExcel


class MainExcel():
    def __init__(self):
        self.app = "Excel.Application"

    def fake2excel(self, columns, rows, path, language):
        """
        @Author & Date  : CoderWanFeng 2022/5/13 0:12
        @Desc  : columns:list，每列的数据名称，默认是名称
                rows：多少行，默认是1
                language：什么语言，可以填english，默认是中文
                path：输出excel的位置，有默认值
        @Ref  : 可以fake的数据类型有：https://mp.weixin.qq.com/s/xVwEjXu58WovgSi4ZTtVQw
        """
        if rows == 0:
            pd.DataFrame().to_excel(str(Path(path).absolute()))
        else:
            # 可以选择英语
            language = 'en_US' if language.lower() == 'english' else 'zh_CN'
            fake = Faker(language)
            excel_dict = {}
            for column in simple_progress(columns, desc=f'columns'):
                excel_dict[column] = list()
                for _ in simple_progress(range(0, rows), desc='rows'):
                    excel_dict[column].append(eval('fake.{func}()'.format(func=column)))
            # 用pandas，将模拟数据，写进excel里面
            res_excel_file = pd.ExcelWriter(str(Path(path).absolute()))
            res_data = pd.DataFrame(excel_dict)
            res_data = pandas_mem.reduce_pandas_mem_usage(res_data)
            res_data.to_excel(res_excel_file, index=False)
            # writer.save()
            res_excel_file.close()

    def merge2excel(self, dir_path, output_file, xlsxSuffix=".xlsx"):
        """
        :param dir_path: 存放excel文件的位置
        :param output_file: 输出合并后excel文件的位置
        :return: 没有返回值
        """
        abs_output_path = Path(output_file).absolute()
        mkdir(abs_output_path.parent)  # 创建输出路径
        if not output_file.endswith(xlsxSuffix):
            raise Exception(f'您自定义的输出文件名，不是以{xlsxSuffix}结尾的')
        file_path_dict = self.getfile(dir_path)  # excel文件所在的文件夹
        try:
            writer = pd.ExcelWriter(output_file)  # 合并后的excel名称
        except PermissionError:
            raise Exception(f'小可爱，你的输出文件，是不是上次打开了没关闭呀？这是你自己指定的输出文件名称：{output_file}')
        for file, path in file_path_dict.items():
            if file.endswith("xlsx"):
                df = pd.read_excel(path)
            if file.endswith("csv"):
                df = pd.read_csv(path)
            df.to_excel(writer, sheet_name=file.split('.')[0], index=False)
        print(f'您指定的Excel文件已经合并完毕，合并后的文件名是{output_file}')
        writer._save()

    def getfile(self, dirpath):
        path = Path(dirpath)
        file_path_dict = {}
        for root, dirs, files in os.walk(dirpath):
            for file in files:
                if file.endswith("xlsx") or file.endswith("csv"):
                    file_path_dict[file] = (path / file)
        return file_path_dict

    def sheet2excel(self, file_path, output_path: str):
        # 先读取一次文件，获取sheet表的名称

        origin_excel = load_workbook(filename=file_path)  # 读取原excel文件
        origin_sheet_names = origin_excel.sheetnames  # 获取sheet的名称
        print(f'一共有{len(origin_sheet_names)}个sheet，名称分别为：{origin_sheet_names}')
        print('拆分开始')

        if len(origin_sheet_names) > 1:  # 如果sheetnames小于1，报错：该文件不需要拆分

            for j in tqdm(range(len(origin_sheet_names))):

                wb = load_workbook(filename=file_path)  # 再读取一次文件，由于每次删除后需要保存一次，所以不能与上一次一样
                sheet = wb[origin_sheet_names[j]]
                wb.copy_worksheet(sheet)

                new_filename = Path(output_path).joinpath(origin_sheet_names[j] + '.xlsx')  # 新建一个sheet命名的excel文件

                for i in tqdm(range(len(origin_sheet_names))):
                    sheet1 = wb[origin_sheet_names[i]]
                    wb.remove(sheet1)

                wb.save(filename=new_filename)

                # 由于使用copy_worksheet后，sheet表名有copy字段，这里做个调整

                new = load_workbook(filename=new_filename)
                news = new.active
                news.title = origin_sheet_names[j]
                new.save(filename=new_filename)
            print('拆分结束')
        else:
            raise Exception(f"你的文件只有一个sheet，难道还要拆分吗？我做不到啊~~~，你的文件名{file_path}")

    def merge2sheet(self, dir_path, output_sheet_name: str, output_excel_name):
        # 多个excel文件，合并到一个sheet里面
        for root, dirs, files in os.walk(dir_path):
            path = Path(dir_path)
            print(f'正在合并的文件有：{files}')
            print(f'合并后的文件名是：{output_excel_name}')
            print(f'合并后的sheet名是：{output_sheet_name}')
            df_list = []
            for file in files:
                if file.endswith("xlsx") or file.endswith("xls"):
                    excel_path = (path / file)
                    df_list.append(pd.read_excel(excel_path))
            res = pd.concat(df_list)
            res.to_excel(
                (path / (output_excel_name + '.xlsx')),
                sheet_name=output_sheet_name,
                index=False  # 不保留index
            )

            pass

    def find_excel_data(self, search_key, target_dir):
        """
        检索指定目录下的excel文件和过滤
        参数：
            search_key：检索的关键词
            target_dir：目标文件夹
        """
        print('该方法已过期，请调用：search4file.find_excel_data(search_key, target_dir)')

    def split_excel_by_column(self, filepath, column, worksheet_name):
        SplitExcel.split_excel_by_column(filepath, column, worksheet_name)

    def excel2pdf(self, excel_path, pdf_path, sheet_id):
        """
        https://blog.csdn.net/qq_57187936/article/details/125605967
        """
        abs_excel_path = str(Path(excel_path).absolute())
        input_excel_path_list1 = get_files(abs_excel_path, suffix='.xls')
        input_excel_path_list2 = get_files(abs_excel_path, suffix='.xlsx')
        input_excel_path_list1.extend(input_excel_path_list2)
        output_pdf_path = Path(pdf_path).absolute()
        mkdir(output_pdf_path)
        for excel_file in input_excel_path_list1:
            with xw.App() as app:  # 下列来源：https://www.qiniu.com/qfans/qnso-57724345#comments
                app.visible = False
                # Initialize new excel workbook
                book = app.books.open(str(excel_file))
                sheet = book.sheets[sheet_id]
                # Construct path for pdf file
                pdf_path_name = os.path.join(str(output_pdf_path), Path(excel_file).stem + '.pdf')
                sheet.to_pdf(path=pdf_path_name, show=False)

    def count4page(self, input_path):
        """
        统计Excel文件打印的页数
        :author Cai-cy
        :param input_path:
        :return:
        """
        # # 指定文件夹路径
        # # 打开 Excel 应用程序
        # excel = win32com.client.Dispatch(self.app)
        #
        # # 遍历文件夹下的所有文件
        # for file_name in os.listdir(input_path):
        #     # 判断文件是否是 Excel 文件
        #     if file_name.endswith(".xlsx") or file_name.endswith(".xls"):
        #         # 打开 Excel 文件
        #         file_path = os.path.join(input_path, file_name)
        #         workbook = excel.Workbooks.Open(file_path)
        #         # 获取 Excel 文件的打印页数
        #         page_count = workbook.ActiveSheet.PageSetup.Pages.Count
        #         # 输出 Excel 文件的打印页数
        #         print(f"{file_name}: {page_count}页")
        #         # 关闭 Excel 文件
        #         workbook.Close()
        #
        # # 关闭 Excel 应用程序
        # excel.Quit()
        pass



#!/usr/bin/env python
# -*- coding:utf-8 -*-
'''
@学习网站      ：www.python-office.com
@读者群     ：http://www.python4office.cn/wechat-group/
@作者  ：B站/抖音/微博/小红书/公众号，都叫：程序员晚枫，微信：CoderWanFeng
@代码日期    ：2024/1/9 22:56
@本段代码的视频说明     ：
'''
from poexcel.core import QueryExcel
#############################################
# File Name: excel.py
# Mail: 1957875073@qq.com
# Created Time:  2022-4-25 10:17:34
# Description: 有关 excel 的自动化操作
#############################################
from poexcel.core.ExcelType import MainExcel

mainExcel = MainExcel()


# todo:输出文件路径
# @except_dec()
def fake2excel(columns=['name'], rows=1, path='./fake2excel.xlsx', language='zh_CN'):
    """
    视频：https://www.bilibili.com/video/BV1wr4y1b7uk/
    演示代码：
    :param columns:
    :param rows:
    :param path:
    :param language:
    :return:
    """
    mainExcel.fake2excel(columns, rows, path, language)


# 多个excel，合并到一个excel的不同sheet中
# @except_dec()
def merge2excel(dir_path, output_file='merge2excel.xlsx'):
    """
    视频：https://www.bilibili.com/video/BV1714y147Ao/
    演示代码：
    :param dir_path:
    :param output_file:
    :return:
    """
    mainExcel.merge2excel(dir_path, output_file)


# 同一个excel里的不同sheet，拆分为不同的excel文件
# @except_dec()
def sheet2excel(file_path, output_path='./'):
    """
    视频：https://www.bilibili.com/video/BV1714y147Ao/
    演示代码：
    :param file_path:
    :param output_path:
    :return:
    """
    mainExcel.sheet2excel(file_path, output_path)


# 搜索excel中指定内容的文件、行数、内容详情
# PR内容 & 作者：https://gitee.com/CoderWanFeng/python-office/pulls/10
# @except_dec()
def find_excel_data(search_key: str, target_dir: str):
    """
    视频：https://www.bilibili.com/video/BV1Bd4y1B7yr/
    演示代码：

    :param search_key:
    :param target_dir:
    :return:
    """
    mainExcel.find_excel_data(search_key, target_dir)


def excel2pdf(excel_path, pdf_path, sheet_id: int = 0):
    """
    将指定的Excel文件的指定工作表转换为PDF文件。
    视频：https://www.bilibili.com/video/BV1A84y1N7or/

    :param excel_path: str, Excel文件的路径。
    :param pdf_path: str, 转换后生成的PDF文件的路径。
    :param sheet_id: int, 工作表的索引，默认为0，表示第一个工作表。
    :return: None
    """
    mainExcel.excel2pdf(excel_path, pdf_path, sheet_id)


def query4excel(query_content, query_path, output_path=None, output_name='output_path/query4excel.xlsx'):
    """
    视频：https://www.bilibili.com/video/BV1Hs4y1S7TT/
    演示代码：

    :param query_content:
    :param query_path:
    :param output_path:
    :param output_name:
    :return:
    """
    QueryExcel.query4excel(query_content, query_path, output_path, output_name)


def count4page(input_path):
    """
    文档：https://blog.csdn.net/weixin_42321517/article/details/131218163
    演示代码：

    :param input_path:
    :return:
    """
    mainExcel.count4page(input_path)


# @except_dec()
def merge2sheet(dir_path, output_sheet_name: str = 'Sheet1', output_excel_name: str = 'merge2sheet'):
    """
    视频：
    演示代码：

    :param dir_path:
    :param output_sheet_name:
    :param output_excel_name:
    :return:
    """
    mainExcel.merge2sheet(dir_path, output_sheet_name, output_excel_name)


# 按指定列的内容，拆分excel
# PR内容 & 作者：：https://gitee.com/CoderWanFeng/python-office/pulls/11
# @except_dec()
def split_excel_by_column(filepath: str, column: int, worksheet_name: str = None):
    """
    视频：
    演示代码：
    :param filepath: 必填，Excel文件的位置和名称
    :param column: 必填，根据第几列拆分
    :param worksheet_name: 选填，可以指定拆分哪一个sheet，不填则默认第一个
    :return:
    """
    mainExcel.split_excel_by_column(filepath, column, worksheet_name)



import unittest
from poexcel.api.excel import *


class TestExcel(unittest.TestCase):
    def test_fake2excel(self):
        fake2excel(language='fdsa', columns=['name', 'text'], rows=200)

    def test_split_excel_by_column(self):
        split_excel_by_column(filepath=r'../../contributors/bulabean/sedemo.xls',
                              column=6)

    def test_sheet2excel(self):
        sheet2excel(file_path=r'/tests/excel_files/fake2excel.xlsx',
                    output_path=r'D:\workplace\code\github\poexcel\tests\output_path')

    def test_merge2sheet(self):
        merge2sheet(dir_path=r'D:\workplace\code\github\python-office\tests\test_files\excel\merge2sheet')

    def test_merge2excel(self):
        merge2excel(dir_path=r'../../contributors/bulabean', output_file='test_merge2excel.xlsx', )

    def test_find_excel_data(self):
        find_excel_data(search_key='刘家站垦殖场', target_dir=r'../../contributors/bulabean')

    def test_query4excel(self):
        # query4excel(query_content='程序员晚枫', query_path=r'D:\test\py310\excel_test')
        query4excel(query_content='程序员晚枫', query_path=r'D:\test\py310\excel_test\course',
                    output_path=r'D:\test\py310\excel_test\output_path', output_name='晚枫')

    def test_excel2pdf(self):
        excel2pdf(excel_path=r'./excel_files/fake2excel.xlsx', pdf_path=r'./output_path')
